@@ -6,6 +6,7 @@ from pymatgen.electronic_structure.core import Spin, Orbital
 from pychemia.code.vasp.doscar import VaspDoscar
 from scipy.ndimage.filters import gaussian_filter1d
 from scipy.ndimage import gaussian_filter
+from scipy.interpolate import interp2d
 from functools import reduce
 import numpy as np
 import pandas as pd
@@ -29,35 +30,55 @@ class Dos:
             by passing 'add' or 'sub'. It spin is passed as 'up' or 'down' this option is ignored.
     """
 
-    def __init__(self, folder, spin='up', combination_method="add"):
+    def __init__(self, folder, spin='up', combination_method="add", shift_efermi=0):
         self.folder = folder
         self.spin = spin
         self.combination_method = combination_method
         self.forbitals = False
-        if os.path.isfile(os.path.join(folder, 'dos.npy')) and os.path.isfile(os.path.join(folder, 'projected_dos.npy')):
-            with open(os.path.join(folder, 'dos.npy'), 'rb') as dos_file:
-                dos = np.load(dos_file)
-            with open(os.path.join(folder, 'projected_dos.npy'), 'rb') as projected_dos_file:
-                projected_dos = np.load(projected_dos_file)
-
-            self.doscar = {
-                'total': dos,
-                'projected': projected_dos,
-            }
+        self.incar = Incar.from_file(
+            os.path.join(folder, 'INCAR')
+        )
+        if 'LORBIT' in self.incar:
+            if self.incar['LORBIT'] >= 11:
+                self.lorbit = True
+            else:
+                self.lorbit = False
         else:
-            self.doscar = VaspDoscar.parse_doscar(os.path.join(folder, 'DOSCAR'))
-            np.save(os.path.join(folder, 'dos.npy'), self.doscar['total'])
-            np.save(os.path.join(folder, 'projected_dos.npy'), self.doscar['projected'])
+            self.lorbit = False
 
-        self.efermi = float(os.popen(f'grep E-fermi {os.path.join(folder, "OUTCAR")}').read().split()[2])
+        if self.lorbit:
+            if os.path.isfile(os.path.join(folder, 'dos.npy')) and os.path.isfile(os.path.join(folder, 'projected_dos.npy')):
+                with open(os.path.join(folder, 'dos.npy'), 'rb') as dos_file:
+                    dos = np.load(dos_file)
+                with open(os.path.join(folder, 'projected_dos.npy'), 'rb') as projected_dos_file:
+                    projected_dos = np.load(projected_dos_file)
+
+                self.doscar = {
+                    'total': dos,
+                    'projected': projected_dos,
+                }
+            else:
+                self.doscar = VaspDoscar.parse_doscar(os.path.join(folder, 'DOSCAR'))
+                np.save(os.path.join(folder, 'dos.npy'), self.doscar['total'])
+                np.save(os.path.join(folder, 'projected_dos.npy'), self.doscar['projected'])
+        else:
+            if os.path.isfile(os.path.join(folder, 'dos.npy')):
+                with open(os.path.join(folder, 'dos.npy'), 'rb') as dos_file:
+                    dos = np.load(dos_file)
+
+                self.doscar = {
+                    'total': dos,
+                }
+            else:
+                self.doscar = VaspDoscar.parse_doscar(os.path.join(folder, 'DOSCAR'))
+                np.save(os.path.join(folder, 'dos.npy'), self.doscar['total'])
+
+        self.efermi = float(os.popen(f'grep E-fermi {os.path.join(folder, "OUTCAR")}').read().split()[2]) + shift_efermi
         self.poscar = Poscar.from_file(
             os.path.join(folder, 'POSCAR'),
             check_for_POTCAR=False,
             read_velocities=False
         )
-        self.incar = Incar.from_file(
-            os.path.join(folder, 'INCAR')
-        )
         self.color_dict = {
             0: '#FF0000',
             1: '#0000FF',
@@ -119,7 +140,9 @@ class Dos:
 
         self.spin_dict = {'up': Spin.up, 'down': Spin.down}
         self.tdos_array = self._load_tdos()
-        self.pdos_array = self._load_pdos()
+
+        if self.lorbit:
+            self.pdos_array = self._load_pdos()
 
     def _load_tdos(self):
         """
@@ -194,6 +217,11 @@ class Dos:
             if self.combination_method == 'sub':
                 pdos = pdos_up - pdos_down
 
+        tdos = self.tdos_array[:,-1]
+        summed_pdos = np.sum(np.sum(pdos, axis=1), axis=1)
+        scale_factor = np.nan_to_num(np.divide(tdos, summed_pdos))
+        pdos = np.multiply(pdos, scale_factor[:,None,None])
+
         return pdos
 
 
@@ -388,31 +416,41 @@ class Dos:
 
         return _smeared_dos
 
-    def _set_density_lims(self, ax, tdensity, tenergy, erange, energyaxis, spin, partial=False, is_dict=False, idx=None, multiple=False):
+    def _set_density_lims(self, ax, tdensity, tenergy, erange, energyaxis, spin, partial=False, is_dict=False, idx=None, multiple=False, log_scale=False):
         energy_in_plot_index = np.where(
             (tenergy >= erange[0]) & (tenergy <= erange[1])
         )[0]
 
         tdensity = tdensity[energy_in_plot_index]
 
-        if len(tdensity.shape) == 1:
-           density_in_plot = tdensity 
+        if len(np.squeeze(tdensity).shape) == 1:
+           density_in_plot = np.squeeze(tdensity) 
         else:
             if spin == 'up' or spin == 'both':
                 max_index = np.argmax(np.max(tdensity, axis=0))
                 density_in_plot = tdensity[:,max_index]
+            else:
+                min_index = np.argmin(np.min(tdensity, axis=0))
+                density_in_plot = tdensity[:,min_index]
 
         if len(ax.lines) == 0:
             if energyaxis == 'y':
                 ax.set_ylim(erange)
                 if spin == 'up' or spin == 'both':
                     ax.set_xlim(0, np.max(density_in_plot) * 1.1)
+                    if log_scale:
+                         ax.set_xlim(np.min(density_in_plot), np.max(density_in_plot) + np.abs(np.max(density_in_plot) * 0.1))
+                    else:
+                        ax.set_xlim(0, np.max(density_in_plot) * 1.1)
                 elif spin == 'down':
                     ax.set_xlim(np.min(density_in_plot) * 1.1, 0)
             elif energyaxis == 'x':
                 ax.set_xlim(erange)
                 if spin == 'up' or spin == 'both':
-                    ax.set_ylim(0, np.max(density_in_plot) * 1.1)
+                    if log_scale:
+                         ax.set_ylim(np.min(density_in_plot), np.max(density_in_plot) + np.abs(np.max(density_in_plot) * 0.1))
+                    else:
+                        ax.set_ylim(0, np.max(density_in_plot) * 1.1)
                 elif spin == 'down':
                     ax.set_ylim(np.min(density_in_plot) * 1.1, 0)
         elif len(ax.lines) > 0:
@@ -519,6 +557,19 @@ class Dos:
 
 
     def _plot_projected_general(self, ax, energy, projected_data, colors, sigma, erange, linewidth, alpha_line, alpha, fill, energyaxis, total):
+        energy_in_plot_index = np.where(
+            (energy >= erange[0] - 0.5) & (energy <= erange[1] + 0.5)
+        )[0]
+        energy = energy[energy_in_plot_index]
+        projected_data = projected_data[energy_in_plot_index]
+
+        if sigma > 0:
+            for i in range(projected_data.shape[-1]):
+                projected_data[:,i] = self._smear(
+                    projected_data[:,i],
+                    sigma=sigma,
+                )
+
         if total:
             self.plot_plain(
                 ax=ax,
@@ -542,13 +593,8 @@ class Dos:
             )
 
         for i in range(projected_data.shape[-1]):
-            if sigma > 0:
-                pdensity = self._smear(
-                    projected_data[:,i],
-                    sigma=sigma,
-                )
-            else:
-                pdensity = projected_data[:,i]
+
+            pdensity = projected_data[:,i]
 
             if energyaxis == 'y':
                 ax.plot(
@@ -604,6 +650,10 @@ class Dos:
         """
 
         tdos_array = self.tdos_array
+        energy_in_plot_index = np.where(
+            (tdos_array[:,0] >= erange[0] - 0.5) & (tdos_array[:,0] <= erange[1] + 0.5)
+        )[0]
+        tdos_array = tdos_array[energy_in_plot_index]
 
         if sigma > 0:
             tdensity = self._smear(
@@ -658,7 +708,7 @@ class Dos:
                     alpha=alpha,
                 )
 
-    def plot_ldos(self, ax, layers, linewidth=1.5, fill=False, alpha=0.3, alpha_line=1.0, sigma=0.05, energyaxis='x', color='black', erange=[-6, 6]):
+    def plot_ldos(self, ax, layers, linewidth=1.5, fill=False, alpha=0.3, alpha_line=1.0, sigma=0.05, energyaxis='x', color='black', log_scale=False, erange=[-6, 6]):
         """
         This function plots the total density of states
 
@@ -685,6 +735,13 @@ class Dos:
         else:
             tdensity = self._sum_layers(layers=layers)
 
+        if log_scale:
+            tdensity = np.log10(tdensity)
+            tdensity = np.log10(tdensity)
+            neg_inf_loc = np.isin(tdensity, -np.inf)
+            min_val = np.min(tdensity[np.logical_not(neg_inf_loc)])
+            tdensity[neg_inf_loc] = min_val
+
         self._set_density_lims(
             ax=ax,
             tdensity=tdensity,
@@ -894,7 +951,7 @@ class Dos:
             self._add_legend(ax, names=[self.orbital_labels[i] for i in orbitals], colors=colors)
 
 
-    def plot_atoms(self, ax, atoms, fill=True, alpha=0.3, alpha_line=1.0, linewidth=1.5, sigma=0.05, energyaxis='y', color_list=None, legend=True, total=True, erange=[-6, 6]):
+    def plot_atoms(self, ax, atoms, fill=True, alpha=0.3, alpha_line=1.0, linewidth=1.5, sigma=0.05, energyaxis='y', color_list=None, legend=True, total=True, erange=[-6, 6], sum_atoms=False):
         """
         This function plots the total density of states with the projected density of states on the given atoms.
 
@@ -921,6 +978,10 @@ class Dos:
 
         projected_data = self._sum_atoms(atoms=atoms)
 
+        if sum_atoms:
+            projected_data = np.sum(projected_data, axis=1).reshape(-1,1)
+            colors = [colors[0]]
+
         self._plot_projected_general(
             ax=ax,
             energy=self.tdos_array[:,0],
@@ -1187,7 +1248,7 @@ class Dos:
             self,
             ax,
             cmap='magma',
-            sigma=5,
+            sigma=0.05,
             energyaxis='y',
             erange=[-6, 6],
             lrange=None,
@@ -1224,7 +1285,21 @@ class Dos:
         atom_densities = self._sum_atoms(atoms=None)[ind]
         densities = np.vstack([np.sum(np.vstack(atom_densities[:,[group]]), axis=1) for group in groups])
         densities = np.transpose(densities)
-        densities = gaussian_filter(densities, sigma=sigma)
+
+        if lrange is not None:
+            atom_index = atom_index[lrange[0]:lrange[1]+1]
+            densities = densities[:, lrange[0]:lrange[1]+1]
+
+        if sigma > 0:
+            for i in range(densities.shape[-1]):
+                densities[:,i] = self._smear(
+                    densities[:,i],
+                    sigma=sigma,
+                )
+
+        f = interp2d(atom_index, energies, densities, kind='cubic')
+        atom_index = np.arange(np.min(atom_index), np.max(atom_index), 0.1)
+        densities = f(atom_index, energies)
 
         if log_scale:
             if np.min(densities) == 0:
@@ -1237,10 +1312,6 @@ class Dos:
         else:
             norm = colors.Normalize(vmin=np.min(densities), vmax=np.max(densities))
 
-        if lrange is not None:
-            atom_index = atom_index[lrange[0]:lrange[1]+1]
-            densities = densities[:, lrange[0]:lrange[1]+1]
-
 
         if log_scale:
             lev_exp = np.arange(
